@@ -43,6 +43,8 @@ import android.view.Surface;
 import android.view.TextureView;
 import android.view.View;
 import android.view.ViewGroup;
+import android.view.animation.Animation;
+import android.view.animation.AnimationUtils;
 import android.widget.FrameLayout;
 import android.widget.ImageView;
 
@@ -53,6 +55,8 @@ import androidx.core.app.ActivityOptionsCompat;
 import androidx.core.content.ContextCompat;
 import androidx.core.graphics.drawable.DrawableCompat;
 import androidx.recyclerview.widget.RecyclerView;
+import androidx.vectordrawable.graphics.drawable.Animatable2Compat;
+import androidx.vectordrawable.graphics.drawable.AnimatedVectorDrawableCompat;
 
 import com.bumptech.glide.load.resource.bitmap.CenterInside;
 import com.bumptech.glide.load.resource.bitmap.RoundedCorners;
@@ -70,6 +74,7 @@ import cx.ring.R;
 import cx.ring.client.MediaViewerActivity;
 import cx.ring.conversation.ConversationPresenter;
 import cx.ring.fragments.ConversationFragment;
+import cx.ring.model.Account;
 import cx.ring.model.CallContact;
 import cx.ring.model.ContactEvent;
 import cx.ring.model.DataTransfer;
@@ -81,7 +86,6 @@ import cx.ring.model.TextMessage;
 import cx.ring.service.DRingService;
 import cx.ring.utils.AndroidFileUtils;
 import cx.ring.utils.ContentUriHandler;
-import cx.ring.utils.FileUtils;
 import cx.ring.utils.GlideApp;
 import cx.ring.utils.GlideOptions;
 import cx.ring.utils.ResourceMapper;
@@ -107,7 +111,11 @@ public class ConversationAdapter extends RecyclerView.Adapter<ConversationViewHo
 
     private int expandedItemPosition = -1;
     private int lastDeliveredPosition = -1;
+    private int lastDisplayedPosition = -1;
     private Observable<Long> timestampUpdateTimer;
+    private int lastMsgPos = -1;
+
+    private boolean isComposing = false;
 
     private static int[] msgBGLayouts = new int[] {
             R.drawable.textmsg_bg_out_first,
@@ -182,7 +190,7 @@ public class ConversationAdapter extends RecyclerView.Adapter<ConversationViewHo
     public void remove(Interaction e) {
         for (int i = mInteractions.size() - 1; i >= 0; i--) {
             Interaction element = mInteractions.get(i);
-            if (e.getId().equals(element.getId())) {
+            if (e.getId() == element.getId()) {
                 mInteractions.remove(i);
                 notifyItemRemoved(i);
                 if (i > 0) {
@@ -205,16 +213,21 @@ public class ConversationAdapter extends RecyclerView.Adapter<ConversationViewHo
 
     @Override
     public int getItemCount() {
-        return mInteractions.size();
+        return mInteractions.size() + (isComposing ? 1 : 0);
     }
 
     @Override
     public long getItemId(int position) {
+        if (isComposing && position == mInteractions.size())
+            return Long.MAX_VALUE;
         return mInteractions.get(position).getId();
     }
 
     @Override
     public int getItemViewType(int position) {
+        if (isComposing && position == mInteractions.size())
+            return MessageType.COMPOSING_INDICATION.ordinal();
+
         Interaction interaction = mInteractions.get(position);
 
         if (interaction != null) {
@@ -257,12 +270,25 @@ public class ConversationAdapter extends RecyclerView.Adapter<ConversationViewHo
 
     @Override
     public void onBindViewHolder(@NonNull ConversationViewHolder conversationViewHolder, int position) {
+        if (isComposing && position == mInteractions.size()) {
+            configureForTypingIndicator(conversationViewHolder);
+            return;
+        }
+
         Interaction interaction = mInteractions.get(position);
         if (interaction == null)
             return;
 
         conversationViewHolder.compositeDisposable.clear();
 
+        if (position > lastMsgPos) {
+            lastMsgPos = position;
+            Animation animation = AnimationUtils.loadAnimation(
+                    conversationViewHolder.itemView.getContext(), R.anim.fade_in);
+            animation.setStartOffset(150);
+            conversationViewHolder.itemView.startAnimation(animation);
+        }
+
         if (interaction.getType() == (InteractionType.TEXT)) {
             configureForTextMessage(conversationViewHolder, interaction, position);
         } else if (interaction.getType() == (InteractionType.CALL)) {
@@ -296,15 +322,17 @@ public class ConversationAdapter extends RecyclerView.Adapter<ConversationViewHo
             holder.player = null;
         }
         if (holder.mMsgTxt != null) {
-            holder.mMsgTxt.setOnClickListener(null);
             holder.mMsgTxt.setOnLongClickListener(null);
         }
+        if (holder.mItem != null) {
+            holder.mItem.setOnClickListener(null);
+        }
         if (expandedItemPosition == holder.getLayoutPosition()) {
-            holder.mMsgDetailTxt.setVisibility(View.GONE);
+            if (holder.mMsgDetailTxt != null)
+                holder.mMsgDetailTxt.setVisibility(View.GONE);
             expandedItemPosition = -1;
         }
         holder.compositeDisposable.clear();
-        super.onViewRecycled(holder);
     }
 
     public void setPrimaryColor(int color) {
@@ -312,6 +340,32 @@ public class ConversationAdapter extends RecyclerView.Adapter<ConversationViewHo
         notifyDataSetChanged();
     }
 
+    public void setComposingStatus(Account.ComposingStatus composingStatus) {
+        boolean composing = composingStatus == Account.ComposingStatus.Active;
+        if (isComposing != composing) {
+            isComposing = composing;
+            if (composing)
+                notifyItemInserted(mInteractions.size());
+            else
+                notifyItemRemoved(mInteractions.size());
+        }
+    }
+
+    public void setLastDisplayed(Interaction interaction) {
+        Log.w(TAG, "setLastDisplayed " + interaction.getDaemonId());
+        for (int i = mInteractions.size() - 1; i >= 0; i--) {
+            Interaction element = mInteractions.get(i);
+            if (interaction.getId() == element.getId()) {
+                if (lastDisplayedPosition != -1)
+                    notifyItemChanged(lastDisplayedPosition);
+                lastDisplayedPosition = i;
+                notifyItemChanged(i);
+                Log.w(TAG, "new displayed item " + i);
+                break;
+            }
+        }
+    }
+
     private static class RecyclerViewContextMenuInfo implements ContextMenu.ContextMenuInfo {
         RecyclerViewContextMenuInfo(int position, long id) {
             this.position = position;
@@ -416,14 +470,12 @@ public class ConversationAdapter extends RecyclerView.Adapter<ConversationViewHo
             viewHolder.mMsgDetailTxtPerm.setVisibility(View.GONE);
         }
 
+        CallContact contact = interaction.getContact();
         if (interaction.isIncoming()) {
             viewHolder.mAvatar.setImageBitmap(null);
             viewHolder.mAvatar.setVisibility(View.VISIBLE);
-            CallContact contact = interaction.getContact();
             if (contact != null) {
-                viewHolder.mAvatar.setImageDrawable(
-                        conversationFragment.getConversationAvatar(contact.getPrimaryNumber())
-                );
+                viewHolder.mAvatar.setImageDrawable(conversationFragment.getConversationAvatar(contact.getPrimaryNumber()));
             }
         } else {
             switch (interaction.getStatus()) {
@@ -435,6 +487,10 @@ public class ConversationAdapter extends RecyclerView.Adapter<ConversationViewHo
                     viewHolder.mStatusIcon.setVisibility(View.VISIBLE);
                     viewHolder.mStatusIcon.setImageResource(R.drawable.round_highlight_off_24);
                     break;
+                case DISPLAYED:
+                    viewHolder.mStatusIcon.setVisibility(View.VISIBLE);
+                    viewHolder.mStatusIcon.setImageDrawable(conversationFragment.getSmallConversationAvatar(contact.getPrimaryNumber()));
+                    break;
                 default:
                     viewHolder.mStatusIcon.setVisibility(View.VISIBLE);
                     viewHolder.mStatusIcon.setImageResource(R.drawable.baseline_check_circle_24);
@@ -640,6 +696,20 @@ public class ConversationAdapter extends RecyclerView.Adapter<ConversationViewHo
         }
     }
 
+    private void configureForTypingIndicator(@NonNull final ConversationViewHolder viewHolder) {
+        AnimatedVectorDrawableCompat anim = AnimatedVectorDrawableCompat.create(viewHolder.itemView.getContext(), R.drawable.typing_indicator_animation);
+        if (anim != null) {
+            viewHolder.mStatusIcon.setImageDrawable(anim);
+            anim.registerAnimationCallback(new Animatable2Compat.AnimationCallback() {
+                @Override
+                public void onAnimationEnd(Drawable drawable) {
+                    anim.start();
+                }
+            });
+            anim.start();
+        }
+    }
+
     /**
      * Configures the viewholder to display a classic text message, ie. not a call info text message
      *
@@ -657,6 +727,7 @@ public class ConversationAdapter extends RecyclerView.Adapter<ConversationViewHo
             Log.e(TAG, "Invalid contact, not able to display message correctly");
             return;
         }
+        Log.w(TAG, "configureForTextMessage " + position + " " + interaction.getDaemonId() + " " + interaction.getStatus());
 
         convViewHolder.mCid = textMessage.getConversation().getParticipant();
         String message = textMessage.getBody().trim();
@@ -713,9 +784,36 @@ public class ConversationAdapter extends RecyclerView.Adapter<ConversationViewHo
 
         convViewHolder.mMsgTxt.setText(message);
 
+        boolean endOfSeq = msgSequenceType == SequenceType.LAST || msgSequenceType == SequenceType.SINGLE;
         if (textMessage.isIncoming()) {
-            convViewHolder.mAvatar.setImageBitmap(null);
-            convViewHolder.mAvatar.setVisibility(View.VISIBLE);
+            if (endOfSeq) {
+                convViewHolder.mAvatar.setImageDrawable(
+                        conversationFragment.getConversationAvatar(contact.getPrimaryNumber())
+                );
+                convViewHolder.mAvatar.setVisibility(View.VISIBLE);
+            } else {
+                if (position == lastMsgPos - 1 && convViewHolder.mAvatar != null) {
+                    Animation animation = AnimationUtils.loadAnimation(
+                            convViewHolder.mAvatar.getContext(), R.anim.fade_out);
+                    animation.setAnimationListener(new Animation.AnimationListener(){
+                        @Override
+                        public void onAnimationStart(Animation arg0) {
+                        }
+                        @Override
+                        public void onAnimationRepeat(Animation arg0) {
+                        }
+                        @Override
+                        public void onAnimationEnd(Animation arg0) {
+                            convViewHolder.mAvatar.setImageBitmap(null);
+                            convViewHolder.mAvatar.setVisibility(View.INVISIBLE);
+                        }
+                    });
+                    convViewHolder.mAvatar.startAnimation(animation);
+                } else {
+                    convViewHolder.mAvatar.setImageBitmap(null);
+                    convViewHolder.mAvatar.setVisibility(View.INVISIBLE);
+                }
+            }
         } else {
             switch (textMessage.getStatus()) {
                 case SENDING:
@@ -726,6 +824,15 @@ public class ConversationAdapter extends RecyclerView.Adapter<ConversationViewHo
                     convViewHolder.mStatusIcon.setVisibility(View.VISIBLE);
                     convViewHolder.mStatusIcon.setImageResource(R.drawable.round_highlight_off_24);
                     break;
+                case DISPLAYED:
+                    if (lastDisplayedPosition == position) {
+                        convViewHolder.mStatusIcon.setVisibility(View.VISIBLE);
+                        convViewHolder.mStatusIcon.setImageDrawable(conversationFragment.getSmallConversationAvatar(contact.getPrimaryNumber()));
+                    } else {
+                        convViewHolder.mStatusIcon.setVisibility(View.GONE);
+                        convViewHolder.mStatusIcon.setImageDrawable(null);
+                    }
+                    break;
                 default:
                     if (position == lastOutgoingIndex()) {
                         convViewHolder.mStatusIcon.setVisibility(View.VISIBLE);
@@ -733,20 +840,12 @@ public class ConversationAdapter extends RecyclerView.Adapter<ConversationViewHo
                         lastDeliveredPosition = position;
                     } else {
                         convViewHolder.mStatusIcon.setVisibility(View.GONE);
+                        convViewHolder.mStatusIcon.setImageDrawable(null);
                     }
             }
         }
 
-        if (msgSequenceType == SequenceType.LAST || msgSequenceType == SequenceType.SINGLE) {
-            setBottomMargin(convViewHolder.mMsgTxt, 8);
-            if (textMessage.isIncoming()) {
-                convViewHolder.mAvatar.setImageDrawable(
-                        conversationFragment.getConversationAvatar(contact.getPrimaryNumber())
-                );
-            }
-        } else {
-            setBottomMargin(convViewHolder.mMsgTxt, 0);
-        }
+        setBottomMargin(convViewHolder.mMsgTxt, endOfSeq ? 8 : 0);
 
         if (isTimeShown) {
             convViewHolder.compositeDisposable.add(timestampUpdateTimer.subscribe(t -> {
@@ -764,7 +863,7 @@ public class ConversationAdapter extends RecyclerView.Adapter<ConversationViewHo
                 }));
             }
             setItemViewExpansionState(convViewHolder, isExpanded);
-            convViewHolder.mMsgTxt.setOnClickListener((View v) -> {
+            convViewHolder.mItem.setOnClickListener((View v) -> {
                 if (convViewHolder.animator != null && convViewHolder.animator.isRunning()) {
                     return;
                 }
@@ -1053,7 +1152,8 @@ public class ConversationAdapter extends RecyclerView.Adapter<ConversationViewHo
         CONTACT_EVENT(R.layout.item_conv_contact),
         CALL_INFORMATION(R.layout.item_conv_call),
         INCOMING_TEXT_MESSAGE(R.layout.item_conv_msg_peer),
-        OUTGOING_TEXT_MESSAGE(R.layout.item_conv_msg_me);
+        OUTGOING_TEXT_MESSAGE(R.layout.item_conv_msg_me),
+        COMPOSING_INDICATION(R.layout.item_conv_composing);
 
         private final int layout;
 
