@@ -20,15 +20,22 @@
 package cx.ring.fragments;
 
 import android.Manifest;
+import android.animation.LayoutTransition;
 import android.animation.ValueAnimator;
+import android.annotation.SuppressLint;
+import android.app.Activity;
 import android.content.ActivityNotFoundException;
 import android.content.ClipData;
+import android.content.ComponentName;
 import android.content.Context;
 import android.content.Intent;
+import android.content.ServiceConnection;
 import android.content.SharedPreferences;
 import android.content.pm.PackageManager;
-import android.graphics.drawable.BitmapDrawable;
+import android.graphics.Typeface;
+import android.content.res.Resources;
 import android.os.Bundle;
+import android.os.IBinder;
 import android.provider.MediaStore;
 import android.text.Editable;
 import android.text.TextUtils;
@@ -41,7 +48,10 @@ import android.view.MenuItem;
 import android.view.View;
 import android.view.ViewGroup;
 import android.view.inputmethod.EditorInfo;
+import android.widget.ImageView;
+import android.widget.RelativeLayout;
 import android.widget.Spinner;
+import android.widget.TextView;
 import android.widget.Toast;
 
 import com.google.android.material.snackbar.Snackbar;
@@ -53,7 +63,10 @@ import androidx.appcompat.app.AppCompatActivity;
 import androidx.appcompat.view.menu.MenuBuilder;
 import androidx.appcompat.view.menu.MenuPopupHelper;
 import androidx.appcompat.widget.PopupMenu;
+import androidx.appcompat.widget.Toolbar;
 import androidx.core.view.ViewCompat;
+import androidx.fragment.app.Fragment;
+import androidx.fragment.app.FragmentManager;
 import androidx.recyclerview.widget.DefaultItemAnimator;
 
 import java.io.File;
@@ -78,6 +91,7 @@ import cx.ring.conversation.ConversationView;
 import cx.ring.databinding.FragConversationBinding;
 import cx.ring.dependencyinjection.JamiInjectionComponent;
 import cx.ring.interfaces.Colorable;
+import cx.ring.model.Account;
 import cx.ring.model.CallContact;
 import cx.ring.model.Conversation;
 import cx.ring.model.Interaction;
@@ -86,10 +100,13 @@ import cx.ring.model.Phone;
 import cx.ring.model.Error;
 import cx.ring.model.Uri;
 import cx.ring.mvp.BaseSupportFragment;
+import cx.ring.services.LocationSharingService;
 import cx.ring.services.NotificationService;
 import cx.ring.utils.ActionHelper;
 import cx.ring.utils.AndroidFileUtils;
 import cx.ring.utils.ContentUriHandler;
+import cx.ring.utils.DeviceUtils;
+import cx.ring.utils.ConversationPath;
 import cx.ring.utils.MediaButtonsHelper;
 import cx.ring.views.AvatarDrawable;
 import io.reactivex.Completable;
@@ -102,7 +119,7 @@ import static android.app.Activity.RESULT_OK;
 public class ConversationFragment extends BaseSupportFragment<ConversationPresenter> implements
         MediaButtonsHelper.MediaButtonsHelperCallback,
         ConversationView, SharedPreferences.OnSharedPreferenceChangeListener {
-    protected static final String TAG = ConversationFragment.class.getSimpleName();
+    private static final String TAG = ConversationFragment.class.getSimpleName();
 
     public static final int REQ_ADD_CONTACT = 42;
 
@@ -110,6 +127,7 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
     public static final String KEY_ACCOUNT_ID = BuildConfig.APPLICATION_ID + ".ACCOUNT_ID";
     public static final String KEY_PREFERENCE_PENDING_MESSAGE = "pendingMessage";
     public static final String KEY_PREFERENCE_CONVERSATION_COLOR = "color";
+    public static final String EXTRA_SHOW_MAP = "showMap";
 
     private static final int REQUEST_CODE_FILE_PICKER = 1000;
     private static final int REQUEST_PERMISSION_CAMERA = 1001;
@@ -118,13 +136,14 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
     private static final int REQUEST_CODE_CAPTURE_AUDIO = 1004;
     private static final int REQUEST_CODE_CAPTURE_VIDEO = 1005;
 
+    private ServiceConnection locationServiceConnection = null;
+
     private FragConversationBinding binding;
     private MenuItem mAudioCallBtn = null;
     private MenuItem mVideoCallBtn = null;
 
     private View currentBottomView = null;
     private ConversationAdapter mAdapter = null;
-    private NumberAdapter mNumberAdapter = null;
     private int marginPx;
     private int marginPxTotal;
     private final ValueAnimator animation = new ValueAnimator();
@@ -137,11 +156,18 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
     private int mSelectedPosition;
 
     private AvatarDrawable mConversationAvatar;
-    private Map<String, AvatarDrawable> mParticipantAvatars = new HashMap();
+    private final Map<String, AvatarDrawable> mParticipantAvatars = new HashMap<>();
+    private final Map<String, AvatarDrawable> mSmallParticipantAvatars = new HashMap<>();
+    private int mapWidth, mapHeight;
 
     public AvatarDrawable getConversationAvatar(String uri) {
         return mParticipantAvatars.get(uri);
     }
+    public AvatarDrawable getSmallConversationAvatar(String uri) {
+        synchronized (mSmallParticipantAvatars) {
+            return mSmallParticipantAvatars.get(uri);
+        }
+    }
 
     private static int getIndex(Spinner spinner, Uri myString) {
         for (int i = 0, n = spinner.getCount(); i < n; i++)
@@ -190,15 +216,22 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
     }
 
     private void updateListPadding() {
-        if (currentBottomView != null && currentBottomView.getHeight() != 0)
+        if (currentBottomView != null && currentBottomView.getHeight() != 0) {
             setBottomPadding(binding.histList, currentBottomView.getHeight() + marginPxTotal);
+            RelativeLayout.LayoutParams params = (RelativeLayout.LayoutParams) binding.mapCard.getLayoutParams();
+            params.bottomMargin = currentBottomView.getHeight() + marginPxTotal;
+            binding.mapCard.setLayoutParams(params);
+        }
     }
 
     @Nullable
     @Override
     public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
         injectFragment(((JamiApplication) getActivity().getApplication()).getRingInjectionComponent());
-        marginPx = getResources().getDimensionPixelSize(R.dimen.conversation_message_input_margin);
+        Resources res = getResources();
+        marginPx = res.getDimensionPixelSize(R.dimen.conversation_message_input_margin);
+        mapWidth = res.getDimensionPixelSize(R.dimen.location_sharing_minmap_width);
+        mapHeight = res.getDimensionPixelSize(R.dimen.location_sharing_minmap_height);
         marginPxTotal = marginPx;
 
         binding = FragConversationBinding.inflate(inflater, container, false);
@@ -213,8 +246,15 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
             insets.consumeSystemWindowInsets();
             return insets;
         });
-        int paddingTop = binding.conversationLayout.getPaddingTop();
-        ViewCompat.setOnApplyWindowInsetsListener(binding.conversationLayout, (v, insets) -> {
+        View layout = binding.conversationLayout;
+
+        // remove action bar height for tablet layout
+        if (DeviceUtils.isTablet(getContext())) {
+            layout.setPadding(layout.getPaddingLeft(), 0, layout.getPaddingRight(), layout.getPaddingBottom());
+        }
+
+        int paddingTop = layout.getPaddingTop();
+        ViewCompat.setOnApplyWindowInsetsListener(layout, (v, insets) -> {
             v.setPadding(
                     v.getPaddingLeft(),
                     paddingTop + insets.getSystemWindowInsetTop(),
@@ -230,6 +270,14 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
                 .flatMapCompletable(this::sendFile)
                 .doFinally(contentInfo::releasePermission)));
         binding.msgInputTxt.setOnEditorActionListener((v, actionId, event) -> actionSendMsgText(actionId));
+        binding.msgInputTxt.setOnFocusChangeListener((view, hasFocus) -> {
+            if (hasFocus)  {
+                Fragment fragment = getChildFragmentManager().findFragmentById(R.id.mapLayout);
+                if (fragment != null) {
+                    ((LocationSharingFragment) fragment).hideControls();
+                }
+            }
+        });
         binding.msgInputTxt.addTextChangedListener(new TextWatcher() {
             @Override
             public void beforeTextChanged(CharSequence s, int start, int count, int after) {
@@ -243,6 +291,7 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
             public void afterTextChanged(Editable s) {
                 String message = s.toString();
                 boolean hasMessage = !TextUtils.isEmpty(message);
+                presenter.onComposingChanged(hasMessage);
                 if (hasMessage) {
                     binding.msgSend.setVisibility(View.VISIBLE);
                     binding.emojiSend.setVisibility(View.GONE);
@@ -308,6 +357,14 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
         animation.removeAllUpdateListeners();
         binding.histList.setAdapter(null);
         mCompositeDisposable.clear();
+        if (locationServiceConnection != null) {
+            try {
+                requireContext().unbindService(locationServiceConnection);
+            } catch (Exception e) {
+                Log.w(TAG, "Error unbinding service: " + e.getMessage());
+            }
+        }
+        mAdapter = null;
         super.onDestroyView();
     }
 
@@ -339,6 +396,7 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
         presenter.selectFile();
     }
 
+    @SuppressLint("RestrictedApi")
     public void expandMenu(View v) {
         Context context = requireContext();
         PopupMenu popup = new PopupMenu(context, v);
@@ -382,6 +440,9 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
                 case R.id.conv_send_file:
                     selectFile();
                     break;
+                case R.id.conv_share_location:
+                    shareLocation();
+                    break;
             }
             return false;
         });
@@ -390,6 +451,36 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
         menuHelper.show();
     }
 
+    public void shareLocation() {
+        presenter.shareLocation();
+    }
+
+    public void closeLocationSharing(boolean isSharing) {
+        RelativeLayout.LayoutParams params = (RelativeLayout.LayoutParams) binding.mapCard.getLayoutParams();
+        if (params.width != mapWidth) {
+            params.width = mapWidth;
+            params.height = mapHeight;
+            binding.mapCard.setLayoutParams(params);
+        }
+        if (!isSharing)
+            hideMap();
+    }
+
+    public void openLocationSharing() {
+        binding.conversationLayout.getLayoutTransition().enableTransitionType(LayoutTransition.CHANGING);
+        RelativeLayout.LayoutParams params = (RelativeLayout.LayoutParams) binding.mapCard.getLayoutParams();
+        if (params.width != ViewGroup.LayoutParams.MATCH_PARENT) {
+            params.width = ViewGroup.LayoutParams.MATCH_PARENT;
+            params.height = ViewGroup.LayoutParams.MATCH_PARENT;
+            binding.mapCard.setLayoutParams(params);
+        }
+    }
+
+    @Override
+    public void startShareLocation(String accountId, String conversationId) {
+        showMap(accountId, conversationId, true);
+    }
+
     /**
      * Used to update with the past adapter position when a long click was registered
      * @param position
@@ -398,6 +489,42 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
         mSelectedPosition = position;
     }
 
+    @Override
+    public void showMap(String accountId, String contactId, boolean open)  {
+        if (binding.mapCard.getVisibility() == View.GONE) {
+            Log.w(TAG, "showMap " + accountId + " " + contactId);
+
+            FragmentManager fragmentManager = getChildFragmentManager();
+            LocationSharingFragment fragment = LocationSharingFragment.newInstance(accountId, contactId, open);
+            fragmentManager.beginTransaction()
+                    .add(R.id.mapLayout, fragment, "map")
+                    .commit();
+            binding.mapCard.setVisibility(View.VISIBLE);
+        }
+        if (open) {
+            Fragment fragment = getChildFragmentManager().findFragmentById(R.id.mapLayout);
+            if (fragment != null) {
+                ((LocationSharingFragment) fragment).showControls();
+            }
+        }
+    }
+
+    @Override
+    public void hideMap() {
+        if (binding.mapCard.getVisibility() != View.GONE) {
+            binding.mapCard.setVisibility(View.GONE);
+
+            FragmentManager fragmentManager = getChildFragmentManager();
+            Fragment fragment = fragmentManager.findFragmentById(R.id.mapLayout);
+
+            if (fragment != null) {
+                fragmentManager.beginTransaction()
+                        .remove(fragment)
+                        .commit();
+            }
+        }
+    }
+
     public void takePicture() {
         if (!presenter.getDeviceRuntimeService().hasVideoPermission()) {
             requestPermissions(new String[]{Manifest.permission.CAMERA}, JamiApplication.PERMISSIONS_REQUEST);
@@ -552,6 +679,18 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
         mAdapter.remove(element);
     }
 
+    @Override
+    public void setComposingStatus(Account.ComposingStatus composingStatus) {
+        mAdapter.setComposingStatus(composingStatus);
+        if (composingStatus == Account.ComposingStatus.Active)
+            scrollToEnd();
+    }
+
+    @Override
+    public void setLastDisplayed(Interaction interaction) {
+        mAdapter.setLastDisplayed(interaction);
+    }
+
     @Override
     public void shareFile(File path) {
         Context c = getContext();
@@ -635,7 +774,10 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
     }
 
     @Override
-    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
+    public void onCreateOptionsMenu(@NonNull Menu menu, @NonNull MenuInflater inflater) {
+        if (!isVisible()) {
+            return;
+        }
         inflater.inflate(R.menu.conversation_actions, menu);
         mAudioCallBtn = menu.findItem(R.id.conv_action_audiocall);
         mVideoCallBtn = menu.findItem(R.id.conv_action_videocall);
@@ -677,6 +819,35 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
         } catch (Exception e) {
             Log.e(TAG, "Can't load conversation preferences");
         }
+
+        if (locationServiceConnection == null) {
+            locationServiceConnection = new ServiceConnection() {
+                @Override
+                public void onServiceConnected(ComponentName name, IBinder service) {
+                    Log.w(TAG, "onServiceConnected");
+                    LocationSharingService.LocalBinder binder = (LocationSharingService.LocalBinder) service;
+                    LocationSharingService locationService = binder.getService();
+                    ConversationPath path = new ConversationPath(presenter.getPath());
+                    if (locationService.isSharing(path)) {
+                        showMap(accountId, contactUri.getUri(), false);
+                    }
+                    try {
+                        requireContext().unbindService(locationServiceConnection);
+                    } catch (Exception e) {
+                        Log.w(TAG, "Error unbinding service", e);
+                    }
+                }
+
+                @Override
+                public void onServiceDisconnected(ComponentName name) {
+                    Log.w(TAG, "onServiceDisconnected");
+                    locationServiceConnection = null;
+                }
+            };
+
+            Log.w(TAG, "bindService");
+            requireContext().bindService(new Intent(requireContext(), LocationSharingService.class), locationServiceConnection, 0);
+        }
     }
 
     @Override
@@ -695,19 +866,30 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
                 .doOnSuccess(d -> {
                     mConversationAvatar = (AvatarDrawable) d;
                     mParticipantAvatars.put(contact.getPrimaryNumber(),
-                                            new AvatarDrawable((AvatarDrawable) d));
+                            new AvatarDrawable((AvatarDrawable) d));
                 })
                 .flatMapObservable(d -> contact.getUpdatesSubject())
                 .observeOn(AndroidSchedulers.mainThread())
                 .subscribe(c -> {
                     mConversationAvatar.update(c);
                     String uri = contact.getPrimaryNumber();
-                    if (mParticipantAvatars.containsKey(uri)) {
-                        mParticipantAvatars.get(uri).update(c);
-                    }
+                    AvatarDrawable ad = mParticipantAvatars.get(uri);
+                    if (ad != null)
+                        ad.update(c);
                     setupActionbar(contact);
                     mAdapter.setPhoto();
                 }));
+        mCompositeDisposable.add(AvatarFactory.getAvatar(requireContext(), contact, false)
+                .doOnSuccess(d -> mSmallParticipantAvatars.put(contact.getPrimaryNumber(), new AvatarDrawable((AvatarDrawable) d)))
+                .flatMapObservable(d -> contact.getUpdatesSubject())
+                .subscribe(c -> {
+                    synchronized (mSmallParticipantAvatars) {
+                        String uri = contact.getPrimaryNumber();
+                        AvatarDrawable ad = mSmallParticipantAvatars.get(uri);
+                        if (ad != null)
+                            ad.update(c);
+                    }
+                }));
     }
 
     @Override
@@ -718,10 +900,8 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
     @Override
     public void displayNumberSpinner(final Conversation conversation, final Uri number) {
         binding.numberSelector.setVisibility(View.VISIBLE);
-        mNumberAdapter = new NumberAdapter(getActivity(),
-                conversation.getContact(),
-                false);
-        binding.numberSelector.setAdapter(mNumberAdapter);
+        binding.numberSelector.setAdapter(new NumberAdapter(getActivity(),
+                conversation.getContact(), false));
         binding.numberSelector.setSelection(getIndex(binding.numberSelector, number));
     }
 
@@ -771,21 +951,64 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
     }
 
     private void setupActionbar(CallContact contact) {
+        if (!isVisible()) {
+            return;
+        }
+
         ActionBar actionBar = ((AppCompatActivity) requireActivity()).getSupportActionBar();
         if (actionBar == null) {
             return;
         }
+
         Context context = actionBar.getThemedContext();
         String displayName = contact.getDisplayName();
         String identity = contact.getRingUsername();
-        if (identity != null && !identity.equals(displayName)) {
-            actionBar.setSubtitle(identity);
+
+        Activity activity = getActivity();
+        if (activity instanceof HomeActivity) {
+            Toolbar toolbar = getActivity().findViewById(R.id.main_toolbar);
+            TextView title = toolbar.findViewById(R.id.contact_title);
+            TextView subtitle = toolbar.findViewById(R.id.contact_subtitle);
+            ImageView logo = toolbar.findViewById(R.id.contact_image);
+
+            if (!((HomeActivity) activity).isConversationSelected()) {
+                title.setText("");
+                subtitle.setText("");
+                logo.setImageDrawable(null);
+                return;
+            }
+
+            logo.setVisibility(View.VISIBLE);
+            title.setText(displayName);
+            title.setTextSize(15);
+            title.setTypeface(null, Typeface.NORMAL);
+
+            if (identity != null && !identity.equals(displayName)) {
+                subtitle.setText(identity);
+
+                RelativeLayout.LayoutParams params = (RelativeLayout.LayoutParams) title.getLayoutParams();
+                params.addRule(RelativeLayout.ALIGN_TOP, R.id.contact_image);
+                title.setLayoutParams(params);
+            } else {
+                subtitle.setText("");
+
+                RelativeLayout.LayoutParams params = (RelativeLayout.LayoutParams) title.getLayoutParams();
+                params.removeRule(RelativeLayout.ALIGN_TOP);
+                params.addRule(RelativeLayout.CENTER_VERTICAL, RelativeLayout.TRUE);
+                title.setLayoutParams(params);
+            }
+
+            logo.setImageDrawable(mConversationAvatar);
+        } else {
+            if (identity != null && !identity.equals(displayName)) {
+                actionBar.setSubtitle(identity);
+            }
+            actionBar.setTitle(displayName);
+            int targetSize = (int) (AvatarFactory.SIZE_AB * context.getResources().getDisplayMetrics().density);
+            mConversationAvatar.setInSize(targetSize);
+            actionBar.setLogo(null);
+            actionBar.setLogo(mConversationAvatar);
         }
-        actionBar.setTitle(displayName);
-        int targetSize = (int) (AvatarFactory.SIZE_AB * context.getResources().getDisplayMetrics().density);
-        mConversationAvatar.setInSize(targetSize);
-        actionBar.setLogo(null);
-        actionBar.setLogo(mConversationAvatar);
     }
 
     public void blockContactRequest() {
@@ -805,7 +1028,7 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
     }
 
     @Override
-    public void onPrepareOptionsMenu(Menu menu) {
+    public void onPrepareOptionsMenu(@NonNull Menu menu) {
         super.onPrepareOptionsMenu(menu);
         boolean visible = binding.cvMessageInput.getVisibility() == View.VISIBLE;
         if (mAudioCallBtn != null)
@@ -877,21 +1100,31 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
     }
 
     public void handleShareIntent(Intent intent) {
-        String type = intent.getType();
-        if (type == null) {
-            Log.w(TAG, "Can't share with no type");
-            return;
-        }
-        if (type.startsWith("text/plain")) {
-            binding.msgInputTxt.setText(intent.getStringExtra(Intent.EXTRA_TEXT));
-        } else {
-            android.net.Uri uri = intent.getData();
-            ClipData clip = intent.getClipData();
-            if (uri == null && clip != null && clip.getItemCount() > 0)
-                uri = clip.getItemAt(0).getUri();
-            if (uri == null)
+        Log.w(TAG, "handleShareIntent " + intent);
+
+        String action = intent.getAction();
+        if (Intent.ACTION_SEND.equals(action) || Intent.ACTION_SEND_MULTIPLE.equals(action)) {
+            String type = intent.getType();
+            if (type == null) {
+                Log.w(TAG, "Can't share with no type");
                 return;
-            startFileSend(AndroidFileUtils.getCacheFile(requireContext(), uri).flatMapCompletable(this::sendFile));
+            }
+            if (type.startsWith("text/plain")) {
+                binding.msgInputTxt.setText(intent.getStringExtra(Intent.EXTRA_TEXT));
+            } else {
+                android.net.Uri uri = intent.getData();
+                ClipData clip = intent.getClipData();
+                if (uri == null && clip != null && clip.getItemCount() > 0)
+                    uri = clip.getItemAt(0).getUri();
+                if (uri == null)
+                    return;
+                startFileSend(AndroidFileUtils.getCacheFile(requireContext(), uri).flatMapCompletable(this::sendFile));
+            }
+        } else if (Intent.ACTION_VIEW.equals(action)) {
+            ConversationPath path = ConversationPath.fromIntent(intent);
+            if (path != null && intent.getBooleanExtra(EXTRA_SHOW_MAP, false)) {
+                shareLocation();
+            }
         }
     }
 
@@ -909,11 +1142,34 @@ public class ConversationFragment extends BaseSupportFragment<ConversationPresen
         //Use Android Storage File Access to download the file
         Intent downloadFileIntent = new Intent(Intent.ACTION_CREATE_DOCUMENT);
         downloadFileIntent.setType(AndroidFileUtils.getMimeTypeFromExtension(file.getExtension()));
-
         downloadFileIntent.addCategory(Intent.CATEGORY_OPENABLE);
         downloadFileIntent.putExtra(Intent.EXTRA_TITLE,file.getDisplayName());
 
         startActivityForResult(downloadFileIntent, ConversationFragment.REQUEST_CODE_SAVE_FILE);
     }
 
+    @Override
+    public void displayNetworkErrorPanel() {
+        showErrorPanel(R.string.error_no_network, null);
+    }
+
+    public void showErrorPanel(final int textResId, @Nullable View.OnClickListener clickListener) {
+        if (binding.errorMsgPane == null || binding.errorMsgPane.getVisibility() == View.VISIBLE) {
+            return;
+        }
+        binding.errorMsgPane.setVisibility(View.VISIBLE);
+        binding.errorMsgPane.setOnClickListener(clickListener);
+        if (binding.errorMsgTxt != null) {
+            binding.errorMsgTxt.setText(textResId);
+        }
+    }
+
+    @Override
+    public void hideErrorPanel() {
+        if (binding.errorMsgPane == null || binding.errorMsgPane.getVisibility() == View.GONE) {
+            return;
+        }
+        binding.errorMsgPane.setVisibility(View.GONE);
+    }
+
 }
